import java.io.*;
import java_cup.runtime.*;
import java.util.*;
import java.lang.Number;
import java.lang.Object.*;

// **********************************************************************
//
//     Class               Kids
//     --------            ----
//
//     DTDnode:
//     -=-=-=-=-
//
//       [< lists >]
//       ObjectListNode	   sequence of ObjectNodes
//       EltContentNode	   sequence of EltTypeListNodes
//       EltTypeListNode   sequence of ElementNodes
//       AttrListNode	   IdNode, sequence of AttributeNodes
//       EnumListNode	   sequence of IdNodes
//
//       [< containing NO other subclasses >]
//       DTDDocNode	   ObjectListNode
//       ObjectNode	   ContentNode, EntIdNode, ProcInstNode
//       ElementNode	   EltTypeNode, SubContentNode
//       SubContentNode	   EltContentNode, OccurIndicNode
//       AttributeNode	   IdNode, EntIdNode, AttrDefinitionNode
//       EnumTypeNode	   EnumListNode, EntIdNode
//       IdNode		   -- none --
//       EntIdNode	   -- none --
//	 ProcInstNode	   -- none --
//       NDataNode	   -- none --
//       StringLitNode	   -- none --
//
//       [< containing other subclasses >]
//       ContentNode	   (see below)
//       EntityDeclNode    (see below)
//       EltTypeNode	   (see below)
//       AttrDefinitionNode (see below)
//       OccurIndicNode	   (see below)
//       StdTypeNode	   (see below)
//       EnumDefaultNode   (see below)
//
//     ContentNode:
//     -=-=-=-=-=-=-
//       EltSpecNode	   IdNode, EntIdNode, EltContentNode,
//			   OccurIndicNode, EltTypeNode
//       AttrSpecNode	   IdNode, AttrListNode
//       EntitySpecNode	   EntityDeclNode
//       NotationSpecNode  IdNode, StringLitNode
//       CommentNode	   -- none --
//
//     EntityDeclNode:
//     -=-=-=-=-=-=-=-
//       ParamEntNode	   ParsedNode
//       ParsedNode	   ParsedEntityDecl, NDataNode
//       ParsedEntityDecl  (see below)
//
//     ParsedEntityDecl:
//     -=-=-=-=-=-=-=-=-
//       ParsedPubNode	   IdNode, StringLitNode, StringLitNode
//       ParsedSysNode	   IdNode, StringLitNode
//       ParsedPlainNode   IdNode, StringLitNode
//
//     EltTypeNode:
//     -=-=-=-=-=-=-
//       OtherNode	   IdNode, EntIdNode, OccurIndicNode,
//			   EltContentNode
//       PCDataNode	   -- none --
//       EmptyNode	   -- none --
//       AnyNode	   -- none --
//
//     AttrDefinitionNode:
//     -=-=-=-=-=-=-=-=-=-
//       StdDefinitionNode   StdTypeNode, StdDefaultNode
//       EnumDefinitionNode  EnumTypeNode, EnumDefaultNode
//       NotationTypeNode    EnumTypeNode, EnumDefaultNode
//
//     OccurIndicNode:
//     -=-=-=-=-=-=-=-
//       PlusNode	   -- none --
//       TimessNode	   -- none --
//       QMarkNodeNode	   -- none --
//       NothingNode	   -- none --
//
//     StdTypeNode:
//     -=-=-=-=-=-=-
//       CDataNode	   -- none --
//       IdentNode	   -- none --
//       IdRefNode	   -- none --
//       IdRefsNode	   -- none --
//       EntityNode	   -- none --
//       EntitiesNode	   -- none --
//       NmTokenNode	   -- none --
//       NmTokensNode	   -- none --
//       UserDefinedTypeNode   -- none --
//
//     EnumDefaultNode:
//     -=-=-=-=-=-=-=-=-
//       NoDefaultNode	   -- none --
//       StdDefaultNode	   (see below)
//
//     StdDefaultNode:
//     -=-=-=-=-=-=-=-
//       RequiredNode	   -- none --
//       FixedNode	   -- none --
//       ImpliedNode	   -- none --
//
//     ----
//
// **********************************************************************

// **********************************************************************
// DTDNode class (base class for all other kinds of nodes)
// **********************************************************************

public class DTDnode {

    // ************--==== Class Variables ====--************

    DTDDocNode dtdDoc;

    protected static final int DIFF_NODE_TYPES = 3;
    protected static final int MAX_CDATA_LEN = 8;

    public static Vector[] dtdNodes;
    /* Vectors are configured as follows:
     * 
     * dtdNodes[0]  = Vector for EltSpecNodes.
     * dtdNodes[1]  = Vector for AttrListNodes.
     * dtdNodes[2]  = Vector for ParsedNodes (parsed entities)
     *
     * **** */

    // Track number of different 'pieces' of the DTD (used for
    // assignment of reduced tag ids).
    protected static int eltCount;
    protected static int attrCount;
    protected static int entCount;

    // **********--==== End Class Variables ====--**********

    // ----

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public DTDnode() {

	dtdNodes = new Vector[DIFF_NODE_TYPES];
	initDtds();

    } // end constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	dtdDoc.unparse(p, indent);

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	dtdDoc.loadDTDInfo();

    } // end method loadDTDInfo.

    // ----

    /* ==========================================================
     *
     * method initDtds():
     * Initializes the dtdNodes vector.
     *
     * -------- */

    private void initDtds() {

	for (int i = 0; i < DIFF_NODE_TYPES; i++)
	    dtdNodes[i] = new Vector();

    } // end method initDtds.

    /* ==========================================================
     *
     * method echoNodes():
     * Prints all occurences of a particular type of node found in
     * the DTD; the specific type is received as an int parameter.
     * If a PrintWriter is received, then the nodes are echoed to
     * file; otherwise, they're echoed to screen.
     *
     * -------- */

    // To Screen.
    public void echoNodes(int nodeType) {

	switch (nodeType) {
	    case 0:	System.out.println("-- Elements --\n");
			break;
	    case 1:	System.out.println("-- Attributes --\n");
			break;
	    case 2:	System.out.println("-- Parsed Entities --\n");
			break;
	} // end switch.

	Iterator itr = dtdNodes[nodeType].iterator();
	while (itr.hasNext()) {
	    Object obj = itr.next();
	    System.out.print(getStringOf(nodeType, obj, 0));
	    if (nodeType != 1)
	      System.out.println();
 	} // end while.
	 System.out.println();

    } // end method echoNodes (1).

    // ----

    // To file.  **** 06/2002 **** 
    public void echoNodes(int nodeType, PrintWriter p) {

	switch (nodeType) {
	    case 0:	p.println("-- Elements --");
			break;
	    case 1:	p.println("-- Attributes --\n");
			break;
	    case 2:	p.println("-- Parsed Entities --\n");
			break;
	} // end switch.

	Iterator itr = dtdNodes[nodeType].iterator();
	while (itr.hasNext()) {
	    Object obj = itr.next();
	      getStringOf(nodeType, obj, 0, p);
 	} // end while.
	 p.println();

    } // end method echoNodes (2).

    /* ==========================================================
     *
     * method writeMapping():
     * Prints a mapping of all elements and attributes found in
     * the DTD to the specified file (for tag reduction).
     *
     * -------- */

    public void writeMapping(String fileName) {

        // **** open output file for mapping.
        FileWriter mapFile = null;
        try {
            mapFile = new FileWriter(fileName);
        } catch (IOException ex) {
            System.err.println("Mapping File " +
                "could not be created.\n");
            System.exit(-1);
        } // end try-catch.

	try { 

	    // Map elements.
	    Iterator itr = dtdNodes[0].iterator();
	    String tmpStr = null;
	    while (itr.hasNext()) {
		Object obj = itr.next();
		tmpStr = getStringOf(0, obj, 0);
		mapFile.write("e " + tmpStr + "\n", 0, tmpStr.length()+3);
 	    } // end while.

	    // Map Attributes.
	    itr = dtdNodes[1].iterator();
	    tmpStr = null;
	    String tmpStr2 = null;
	    while (itr.hasNext()) {
		Object obj = itr.next();
		for (int i = 0; i < ((AttrListNode)obj).mySize(); i++) {
		  tmpStr = ((AttrListNode)obj).toString(i,null);
		  tmpStr2 = ((AttrListNode)obj).getId(i);
		  mapFile.write("a " + tmpStr + " " + tmpStr2 + "\n", 0,
				tmpStr.length() + tmpStr2.length() + 4);
		} // end for.
 	    } // end while.

	    // Clean up
	    mapFile.close();

	} catch (IOException e) {
	    System.err.println("Error writing mapping file.");
	} // end catch.

    } // end method writeMapping.

    /* ==========================================================
     *
     * method getRandomElement():
     * Receives a Random object and uses it to select one of the
     * elements contained in this DTD; the string representation of
     * that Element is returned.
     *
     * -------- */

    public String getRandomElement(Random rnd) {

	int count = 0;
	int target = rnd.nextInt(dtdNodes[0].size());
	Object obj = null;
	Iterator itr = dtdNodes[0].iterator();

	if (!itr.hasNext())
	// no elements in this DTD.
	    return "UNAVAILABLE";

	// else, assign first element to obj, and start from there.
	obj = itr.next();
	while ((count < target) && itr.hasNext()) {
	    obj = itr.next();
	    count++;
	} // end while.

	if (count == target)
	    return ((EltSpecNode)obj).toString();

	return "UNAVAILABLE";

    } // end method getRandomElement;

    /* ==========================================================
     *
     * method getAttrListFor():
     * Receives a String 'str' and returns the attribute list associated
     * with the element named 'str'.
     *
     * -------- */

    public AttrListNode getAttrListFor(String eltName) {

	Iterator itr = dtdNodes[1].iterator();
	AttrListNode attList = null;
	while (itr.hasNext()) {
	    Object obj = itr.next();
	    String str = ((AttrListNode)obj).toString();
	    if (str.compareTo(eltName) == 0) {
	    // then this is the desired attribute list.
		attList = (AttrListNode)obj;
		break;
	    } // end if.
	} // end while.

	return attList;

    } // end method getAttrListFor.

    /* ==========================================================
     *
     * method getRandomAttribute():
     * Receives a Random object and uses it to select one of the
     * Attributes from the given Element in this DTD; the string
     * representation of that Attribute is returned.
     *
     * -------- */

    public String getRandomAttribute(String eltName, Random rnd) {

	// Get attributes for the received eltName.
	AttrListNode attList = getAttrListFor(eltName);

	if (attList == null)
	// couldn't find element name.
	    return "UNAVAILABLE";

	// Else, get a random attribute from attList.

	if (attList.mySize() == 0)
	// there aren't any attributes for this element.
	    return "UNAVAILABLE";
	// else, get it.
	int target = rnd.nextInt(attList.mySize());
	return attList.toString(target,null);

    } // end method getRandomAttribute

    /* ==========================================================
     *
     * method getRandomEntity():
     * Receives a Random object and uses it to select one of the
     * Entities contained in this DTD; the string representation of
     * that Entity is returned.
     *
     * -------- */

    public String getRandomEntity(Random rnd) {

	int count = 0;

	if (dtdNodes[2].size() == 0)
	// there aren't any entities; return fail.
	    return "UNAVAILABLE";

	// Now, get random target.
	int target = rnd.nextInt(dtdNodes[2].size());

	Object obj = null;
	Iterator itr = dtdNodes[2].iterator();

	// Assign first element to obj, and start from there.
	obj = itr.next();
	while ((count < target) && itr.hasNext()) {
	    obj = itr.next();
	    count++;
	} // end while.

	if (count == target)
		return ((ParsedNode)obj).toString();

	return "UNAVAILABLE";

    } // end method getRandomEntity.

    /* ==========================================================
     *
     * method getRandomCDATA():
     * Receives a Random object and uses it to formulate some
     * random character string that can be used as either CDATA or
     * PCDATA in an XML document.
     *
     * -------- */

    public String getRandomCDATA(Random rnd) {

	// Determine a length (if one wasn't received as a param.
	int length = rnd.nextInt(MAX_CDATA_LEN - 1) + 1;

	// Now form a random string of that length.
	String result = "";
	for (int i = 0; i < length; i++) {
	    // Get decimal value of an ASCII char.
	    char tmp = (char)(rnd.nextInt(93) + 33);
	    result += tmp;
	} // end for.

	return result;

    } // end method getRandomCDATA.

    /* ==========================================================
     *
     * method insertNode():
     * inserts received object into the specified Vector of
     * the dtdNodes structure (IN ORDER of ID LENGTH!).  Note
     * that id's must be ordered by length because when we are
     * are trying to match them (for tag replacement), we want
     * to replace longer names FIRST, in case two id's have the
     * same prefix (for example, "personal" and "person".  Since
     * replacement is done on the first match found in the list,
     * the longest id's must come first.
     *
     * -------- */

    public void insertNode(int eltType, Object newObj) {

	int count = 0;
	Iterator itr = dtdNodes[eltType].iterator();
	String eltString = getStringOf(eltType, newObj, 1);
	while (itr.hasNext()) {
	    Object obj = itr.next();
	    String str = getStringOf(eltType, obj, 1);
	    if (str.length() <= eltString.length())
	    // this our point of insertion.
		break;
	    count++;
	} // end while.

	dtdNodes[eltType].insertElementAt(newObj, count);
	return;

    } // end method insertNode.

    /* ==========================================================
     *
     * method getStringOf():
     * Return string corresponding to received object and eltType.
     * If a PrintWriter object is received, the string is printed
     * to file; otherwise, it is returned to the caller.
     *
     * -------- */

    // To screen.
    public String getStringOf(int eltType, Object obj,
		int silent) {

	switch(eltType) {
//	    case 0: return ((EltSpecNode)obj).getNameId();
	    case 0: return ((EltSpecNode)obj).toString();
	    case 1: if (silent == 0) {
		      System.out.println(((AttrListNode)obj).toString());
		      ((AttrListNode)obj).toString(-1,null);
		      return "";
		    } // end if.
		    else
		      return ((AttrListNode)obj).toString();
	    case 2:
//		    return ((ParsedNode)obj).getNameId();
		    return ((ParsedNode)obj).toString();
	    default: return ("Invalid Vector Number!\n");
	} // end switch,

    } // end method getStringOf.

    // ----

    // To file. **** 06/2002 ****
    public void getStringOf(int eltType, Object obj,
		int silent, PrintWriter p) {

	switch(eltType) {
	    case 0: p.println(((EltSpecNode)obj).toString());
		    break;
	    case 1: if (silent == 0) {
		      ((AttrListNode)obj).toString(-1, p);
		    } // end if.
		    break;
	    case 2:
		    p.println(((ParsedNode)obj).toString());
		    break;
	    default: System.err.println("Invalid vector number!");
		    return;
	} // end switch,

    } // end method getStringOf.

    /* ==========================================================
     *
     * method reducedForm():
     * Returns the reducedTag corresponding to the received
     * Element id (for tag reduction).
     *
     * -------- */

     public String reducedForm(IdNode id) {

	Iterator itr = dtdNodes[0].iterator();
	while (itr.hasNext()) {
	    Object obj = itr.next();
	    String str = ((EltSpecNode)obj).toString();
	    if (str.compareTo(id.toString()) == 0) {
		return (((EltSpecNode)obj).getId());
	    } // end if.
	} // end while.
	return "UNAVAILABLE";

    } // end method reducedForm.

    /* ==========================================================
     *
     * method toBase26()
     * Convert received integer to base 26 (for tag reduction).
     *
     * -------- */

    public String toBase26(int anInt) {

	anInt--;
	int tmp = anInt / 26;
	int digitCnt = 0;
	while (tmp > 0) {
	    digitCnt++;
	    tmp = tmp / 26;
	} // end while.
	digitCnt++;

	String newNumber = "";
	tmp = anInt;
	int tmp2 = 0;
	for (int i = digitCnt; i > 0; i--) {
	   if ((tmp/26) == 0) {
		newNumber += (char)(tmp%26 + 65);
	   } // end if.
	   else {
		newNumber += (char)(tmp/26 + 64);
		tmp = tmp - ((tmp/26) * (i-1) * 26);
	   } // end else.
	} // end for.

	return newNumber;

   } // end method toBase26.

    /* ==========================================================
     *
     * method getDTDInfo():
     * Returns elements, attributes, and entities found from
     * parsing the DTD.
     *
     * -------- */

    public Vector[] getDTDInfo() {

	return dtdNodes;

    } // end method getDTDInfo.

} // end class DTD.

// **********************************************************************
// **********************************************************************
// 'DTDnode' sub-classes: [< Lists >].
// **********************************************************************
// **********************************************************************

// ------------------------------
// -=-=-=- ObjectListNode -=-=-=-
// ------------------------------

class ObjectListNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public ObjectListNode(Sequence S) {

        myObjects = S;

    } // end constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

        try {
            for (myObjects.start(); myObjects.isCurrent();
	      myObjects.advance()) {
                ((ObjectNode)myObjects.getCurrent()).unparse(p, 0);
            }
        } catch (NoCurrentException ex) {
            System.err.println("No Current in ObjectListNode.");
            System.exit(-1);
        } // end try-catch.

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

        try {
            for (myObjects.start(); myObjects.isCurrent();
	      myObjects.advance()) {
                ((ObjectNode)myObjects.getCurrent()).loadDTDInfo();
            }
        } catch (NoCurrentException ex) {
            System.err.println("No Current in ObjectListNode.");
            System.exit(-1);
        } // end try-catch.

    } // end method loadDTDInfo

    // ----
 
    // sequence of kids (ObjectNodes).
    private Sequence myObjects;

} // end class ObjectListNode

// ------------------------------
// -=-=-=- EltContentNode -=-=-=-
// ------------------------------

class EltContentNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public EltContentNode(Sequence S) {

        myContent = S;

    } // end constructor.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

        try {
            for (myContent.start(); myContent.isCurrent();
	      myContent.advance()) {
		((EltTypeListNode)myContent.getCurrent()).loadDTDInfo();
            }
        } catch (NoCurrentException ex) {
            System.err.println("No Current in EltContentNode.");
            System.exit(-1);
        } // end try-catch.

    } // end method loadDTDInfo.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */
 
    public void unparse(PrintWriter p, int indent) {

	boolean first = true;
        try {
            for (myContent.start(); myContent.isCurrent();
	      myContent.advance()) {
		if (!first)
		  p.print("|");
		((EltTypeListNode)myContent.getCurrent()).unparse(p,indent);
		first = false;
            }
        } catch (NoCurrentException ex) {
            System.err.println("No Current in EltContentNode.");
            System.exit(-1);
        } // end try-catch.

    } // end method unparse.

    // ----

    // sequence of kids (EltTypeListNodes).
    private Sequence myContent;

} // end class EltContentNode.

// -------------------------------
// -=-=-=- EltTypeListNode -=-=-=-
// -------------------------------

class EltTypeListNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public EltTypeListNode(Sequence S) {

        myTypes = S;

    } // end constructor.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

        try {
            for (myTypes.start(); myTypes.isCurrent();
	      myTypes.advance()) {
		((ElementNode)myTypes.getCurrent()).loadDTDInfo();
            }
        } catch (NoCurrentException ex) {
            System.err.println("No Current in EltTypeListNode.");
            System.exit(-1);
        } // end try-catch.

    } // end method loadDTDInfo.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */
 
    public void unparse(PrintWriter p, int indent) {

	boolean first = true;
        try {
            for (myTypes.start(); myTypes.isCurrent();
	      myTypes.advance()) {
		if (!first)
		  p.print(",");
		((ElementNode)myTypes.getCurrent()).unparse(p,indent);
		first = false;
            }
        } catch (NoCurrentException ex) {
            System.err.println("No Current in EltTypeListNode.");
            System.exit(-1);
        } // end try-catch.

    } // end method unparse.

    // ----

    // sequence of kids (ElementNodes).
    private Sequence myTypes;

} // end class EltTypeListNode

// ------------------------------
// -=-=-=- AttrListNode -=-=-=-
// ------------------------------

class AttrListNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public AttrListNode(Sequence S, IdNode name) {

        myAttrs = S;
	element = name;

    } // end constructor.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	insertNode(1, this);

    } // end loadDTDInfo.

    // ----
 
    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

        try {
            for (myAttrs.start(); myAttrs.isCurrent();
	      myAttrs.advance()) {
		p.print("\t");
		((AttributeNode)myAttrs.getCurrent()).unparse(p,0);
            }
        } catch (NoCurrentException ex) {
            System.err.println("No Current in AttrListNode (unparse).");
            System.exit(-1);
        } // end try-catch.

    } // end unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * A "toString" call with no parameters will simply return the name
     * of the ELEMENT for which this list contains attributes.
     *
     * -------- */

    public String toString() {

	return element.toString();

    } // end method toString().

    // ----


    /* ==========================================================
     *
     * method toString():
     *
     * A "toString" call with some integer will print the name of
     * the ATTRIBUTE corresponding to that integer in this list of
     * attributes If the received PrintWriter is null, then the
     * name is printed to the screen; otherwise, it is printed to
     * the printwriter p.
     *
     * -------- */

    public String toString(int attrNum, PrintWriter p) {

        try {
	    int cnt = 0;
	    boolean printing = false;
	    if (attrNum == -1) {
	    // then we just want to print things out.
		attrNum = myAttrs.length();
		printing = true;
	    } // end if.
	    myAttrs.start();
	    while ((cnt < attrNum) && myAttrs.isCurrent()) {
	      if (printing) {
		if (p == null) {
		    System.out.println(((AttributeNode)myAttrs.getCurrent()).toString());
//			((AttributeNode)myAttrs.getCurrent()).getNameId());
		} // end inner if.
		else {
		    p.println(((AttributeNode)myAttrs.getCurrent()).toString());
		} // end inner else.
	      } // end outer if.
	      myAttrs.advance();
	      cnt++;
	    } // end while.
	    if (!printing)
	      return ((AttributeNode)myAttrs.getCurrent()).toString();
	    else
	      return "";
        } catch (NoCurrentException ex) {
            System.err.println("No Current in AttrListNode (toString).");
            System.exit(-1);
        } // end try-catch.

	return "UNAVAILABLE";

    } // end method toString().

    // ----

    /* ==========================================================
     *
     * method getId():
     *
     * A "getId" call with some integer will return the reducedTag of
     * the ATTRIBUTE corresponding to that integer in this list of
     * attributes.
     *
     * -------- */

    public String getId(int attrNum) {

        try {
	    int cnt = 0;
	    myAttrs.start();
	    while ((cnt < attrNum) && myAttrs.isCurrent()) {
	      myAttrs.advance();
	      cnt++;
	    } // end while.
	    return ((AttributeNode)myAttrs.getCurrent()).getId();
        } catch (NoCurrentException ex) {
            System.err.println("No Current in AttrListNode (toString).");
            System.exit(-1);
        } // end try-catch.

	return "UNAVAILABLE";

    } // end method getId.

    // ----

    public int mySize() {

	return myAttrs.length();

    } // end method mySize().

    // ----

    // 2 kids (1 is a sequence of kids (AttributeNodes)).
    private Sequence myAttrs;
    private IdNode element;

} // end class AttrListNode.

// ------------------------------
// -=-=-=- EnumListNode -=-=-=-
// ------------------------------

class EnumListNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public EnumListNode(Sequence S) {

        myValues = S;

    } // end constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */
 
    public void unparse(PrintWriter p, int indent) {

	boolean first = true;
        try {
            for (myValues.start(); myValues.isCurrent();
	      myValues.advance()) {
		if (!first)
 		  p.print("|");
		((IdNode)myValues.getCurrent()).unparse(p,indent);
		first = false;
            }
        } catch (NoCurrentException ex) {
            System.err.println("No Current in EnumListNode.");
            System.exit(-1);
        } // end try-catch.

    } // end method unparse.

    // ----

    // sequence of kids (IdNodes).
    private Sequence myValues;

} // end class EnumListNode.

// **********************************************************************
// **********************************************************************
// 'DTDnode' sub-classes: [< Containing NO other subclasses >].
// **********************************************************************
// **********************************************************************

// ------------------------------
// -=-=-=- DTDDocNode -=-=-=-
// ------------------------------

class DTDDocNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public DTDDocNode(ObjectListNode objects) {

	myObjList = objects;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	myObjList.loadDTDInfo();

    } // end method loadDTDInfo.

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

      myObjList.unparse(p, 0);

//      System.out.println();
//      for (int i = 0; i < DIFF_NODE_TYPES; i++)
//	echoNodes(i);

    } // end method unparse.

    // ----

    // 1 kid.
    private ObjectListNode myObjList;

} // end class DTDDocNode.

// ------------------------------
// -=-=-=- ObjectNode -=-=-=-
// ------------------------------

class ObjectNode extends DTDnode {

    /* ==========================================================
     *
     * Constructors.
     *
     * -------- */

    public ObjectNode(ContentNode contents) {

	myContent = contents;
	haveContent = true;
	havepInst = false;

    } // end Constructor 1.

    // ----

    public ObjectNode(ProcInstNode pInst) {

	myPInst = pInst;
	haveContent = false;
	havepInst = true;

    } // end Constructor 2.

    // ----

    public ObjectNode(EntIdNode entityId) {

	myEntId = entityId;
	haveContent = false;
	havepInst = false;

    } // end Constructor 3.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	if (haveContent)
	  myContent.loadDTDInfo();

    } // end method loadDTDInfo.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	if (haveContent) {
	  p.print("<!");
	  myContent.unparse(p, indent);
	  p.println(">");
	} // end if.
	else if (havepInst) {
	  myPInst.unparse(p, indent);
	} // end if.
	else {
	  myEntId.unparse(p, 0);
	  p.println();
	} // end else.

    } // end method unparse.

    // ----

    // 2 kids
    private ContentNode myContent;
    private EntIdNode myEntId;
    private ProcInstNode myPInst;

    private boolean haveContent;
    private boolean havepInst;

    private int myLineNum;    
    private int myColNum;   

} // end class ObjectNode

// ------------------------------
// -=-=-=- ElementNode -=-=-=-
// ------------------------------

class ElementNode extends DTDnode {

    /* ==========================================================
     *
     * Constructors.
     *
     * -------- */

    public ElementNode(EltTypeNode content) {

	myContent = content;
	isSubContent = false;

    } // end Constructor 1.

    // ----

    public ElementNode(SubContentNode content) {

	mySubContent = content;
	isSubContent = true;

    } // end Constructor 2.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	if (isSubContent)
	    mySubContent.loadDTDInfo();
	else
	    myContent.loadDTDInfo();

    } // end method loadDTDInfo.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	if (isSubContent)
	    mySubContent.unparse(p, 0);
	else
	    myContent.unparse(p, 0);

    } // end method unparse.

    // ----

    // 2 kids.
    private EltTypeNode myContent;
    private SubContentNode mySubContent;

    private boolean isSubContent;

} // end class ElementNode

// ------------------------------
// -=-=-=- SubContentNode -=-=-=-
// ------------------------------

class SubContentNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public SubContentNode(EltContentNode sub, OccurIndicNode occur) {

	myContent = sub;
	myOccur = occur;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	myContent.loadDTDInfo();

    } // end method loadDTDInfo.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("(");
	myContent.unparse(p, 0);
	p.print(")");
	myOccur.unparse(p, 0);

    } // end method unparse.

    // ----

    // 2 kids.
    private EltContentNode myContent;
    private OccurIndicNode myOccur;

} // end class SubContentNode

// ------------------------------
// -=-=-=- AttributeNode -=-=-=-
// ------------------------------

class AttributeNode extends DTDnode {

    /* ==========================================================
     *
     * Constructors.
     *
     * -------- */

    public AttributeNode(IdNode attrName,
      AttrDefinitionNode attrDef) {

	myName = attrName;
	myDef = attrDef;
        haveDef = true;

	// Create reduced string for this id.
	reducedTag = toBase26(++attrCount);

    } // end constructor 1.

    // ----

    public AttributeNode(EntIdNode entityId) {

	myEntId = entityId;
        haveDef = false;

	// Create reduced string for this id.
	reducedTag = toBase26(++attrCount);

    } // end constructor 2.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

      if (haveDef) {
//	p.print(reducedTag);
	myName.unparse(p, 0);
        p.print(" ");
        myDef.unparse(p, 0);
      } // end if.
      else
	myEntId.unparse(p, 0);
      p.println();

    } // method unparse.

    // ----

    public String toString() {

	if (haveDef)
	  return (myName.toString() + " " + myDef.getType());
	else
	  return myEntId.toString();

    } // end method toString.

    // ----

    public String getNameId() {

	if (haveDef)
	  return (myName.toString() + " " + reducedTag);
	else
	  return (myEntId.toString());

    } // end method getNameId.

    // ----

    public String getId() {

	return (reducedTag);

    } // end method getId.

    // ----

    // 3 kids.
    private IdNode myName;
    private EntIdNode myEntId;
    private AttrDefinitionNode myDef;

    private String reducedTag;
    private boolean haveDef;

} // end class AttributeNode

// ------------------------------
// -=-=-=- EnumTypeNode -=-=-=-
// ------------------------------

class EnumTypeNode extends DTDnode {

    /* ==========================================================
     *
     * Constructors.
     *
     * -------- */

    public EnumTypeNode(EnumListNode enums) {

	myList = enums;
        haveList = true;

    } // end Constructor 1.

    // ----

    public EnumTypeNode(EntIdNode entityId) {

	myEntId = entityId;
	haveList = false;

    } // end Constructor 2.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

        if (haveList) {
	  p.print("(");
	  myList.unparse(p, 0);
	  p.print(")");
	} // end outer if.
	else
	  myEntId.unparse(p, 0);

    } // end method unparse.

    // ----

    // 2 kids.
    private EnumListNode myList;
    private EntIdNode myEntId;

    private boolean haveList;

} // end class EnumTypeNode

// ------------------------------
// -=-=-=- IdNode -=-=-=-
// ------------------------------

class IdNode extends DTDnode {

    /* ==========================================================
     *
     * Constructors.
     *
     * -------- */

    public IdNode(int lineNum, int colNum, String strVal) {

	myLineNum = lineNum;
	myColNum = colNum;
	myStrVal = strVal;

    } // end Constructor 1.

    // ----

    public IdNode(EntIdNode entId) {

	myLineNum = entId.getLine();
	myColNum = entId.getCol();
	myStrVal = entId.toString();

    } // end Constructor 2.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print(myStrVal);

    } // end method unparse.

    // ----

    public String toString() {

	return myStrVal;

    } // end method toString.

    // ----

    private int myLineNum;
    private int myColNum;
    private String myStrVal;

} // end class IdNode.

// ------------------------------
// -=-=-=- ProcInstNode -=-=-=-
// ------------------------------

class ProcInstNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public ProcInstNode(int lineNum, int colNum, String strVal) {

        myLineNum = lineNum;
        myColNum = colNum;
        myProcInst = strVal;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */
 
    public void unparse(PrintWriter p, int indent) {

        p.println(myProcInst);

    } // end method unparse.

    // ----
    
    private int myLineNum;    
    private int myColNum;   
    private String myProcInst;

} // end class ProcInst.

// ------------------------------
// -=-=-=- EntIdNode -=-=-=-
// ------------------------------

class EntIdNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public EntIdNode(int lineNum, int colNum, String strVal) {

	myLineNum = lineNum;
	myColNum = colNum;
	myStrVal = strVal;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print(myStrVal);

    } // end method unparse.

    // ----

    public String toString() {

	return myStrVal;

    } // end toString.

    // ----

    public int getLine() {

	return myLineNum;

    } // end method getLine.

    // ----

    public int getCol() {

	return myColNum;

    } // end method getCol.

    // ----

    private int myLineNum;
    private int myColNum;
    private String myStrVal;

} // end class EntIdNode.

// ------------------------------
// -=-=-=- NDataNode -=-=-=-
// ------------------------------

class NDataNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public NDataNode(IdNode name) {

	myName = name;

    } // end Constructor.

    // ----

    public void unparse(PrintWriter p, int indent) {

	p.print("NDATA ");
	myName.unparse(p, 0);

    } // end method unparse.

    // ----

    // 1 kid.
    private IdNode myName;

} // end class NDataNode

// ------------------------------
// -=-=-=- StringLitNode -=-=-=-
// ------------------------------

class StringLitNode extends DTDnode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public StringLitNode(int lineNum, int colNum, String strVal) {

        myLineNum = lineNum;
        myColNum = colNum;
        myStrVal = strVal;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */
 
    public void unparse(PrintWriter p, int indent) {

        p.print( myStrVal );

    } // end method unparse.

    // ----

    public String strVal() {

	return myStrVal;

    } // end method strVal.

    // ----

    private int myLineNum;    
    private int myColNum;   
    private String myStrVal;

} // end class StringLitNode.

// **********************************************************************
// **********************************************************************
// 'DTDnode' sub-classes: [< Containing other subclasses >].
// **********************************************************************
// **********************************************************************

// -------------------------------------------------
// -=-=-=- ContentNode and its subclasses -=-=-=-
// -------------------------------------------------

abstract class ContentNode extends DTDnode
{
  public void loadDTDInfo() {};
}

// ========== EltSpecNode ==========

class EltSpecNode extends ContentNode {
    
    /* ==========================================================
     *
     * Constructors.
     *
     * -------- */

    public EltSpecNode(IdNode eltName, EltContentNode contents,
		OccurIndicNode occur) {

	myName = eltName;
	myContents = contents;
	myOccur = occur;
        haveContents = true;
	haveEntId = false;

	// Create reduced string for this id.
	reducedTag = toBase26(++eltCount);

    } // end constructor 1.

    // ----

    public EltSpecNode(IdNode eltName, EltTypeNode eltType) {

	myName = eltName;
	myType = eltType;
	haveContents = false;
	haveEntId = false;

	// Create reduced string for this id.
	reducedTag = toBase26(++eltCount);

    } // end constructor 2.

    // ----

    public EltSpecNode(IdNode eltName, EntIdNode entityId) {

	myName = eltName;
	myEntId = entityId;
	haveContents = false;
	haveEntId = true;

	// Create reduced string for this id.
	reducedTag = toBase26(++eltCount);

    } // end constructor 3.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	insertNode(0, this);

    } // end method loadDTDInfo.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("ELEMENT ");
	myName.unparse(p, 0);
//	p.print(reducedTag);
        if (haveContents) {
	  p.print(" (");
	  myContents.unparse(p, 0);
	  p.print(")");
	  myOccur.unparse(p, 0);
	} // end if.
	else if (haveEntId) {
	  p.print(" ");
	  myEntId.unparse(p, 0);
	} // end else.
	else {
	  p.print(" ");
	  myType.unparse(p, 0);
	} // end else.

    } // end method unparse.

    // ----

    public String toString() {

	return (myName.toString());

    } // end method toString.

    // ----

    public String getNameId() {

	return (myName.toString() + " " + reducedTag);

    } // end method getNameId.

    // ----

    public String getId() {

	return (reducedTag);

    } // end method getId.

    // ----

    // 5 kids.
    private IdNode myName;
    private EntIdNode myEntId;
    private EltContentNode myContents;
    private OccurIndicNode myOccur;
    private EltTypeNode myType;

    private boolean haveContents;
    private boolean haveEntId;
    private String reducedTag;

} // end class EltSpecNode

// ========== AttrSpecNode ==========

class AttrSpecNode extends ContentNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public AttrSpecNode(IdNode eltName, AttrListNode attrList) {

	myElt = eltName;
	myList = attrList;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	myList.loadDTDInfo();

    } // end method loadDTDInfo.

    // ----

    public void unparse(PrintWriter p, int indent) {

	p.print("ATTLIST ");
	myElt.unparse(p, 0);
//	p.println(reducedForm(myElt));
	myList.unparse(p, 0);

    } // end method unparse.

    // ----

    // 2 kids.
    private IdNode myElt;
    private AttrListNode myList;

} // end class AttrSpecNode

// ========== EntitySpecNode ==========

class EntitySpecNode extends ContentNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public EntitySpecNode(EntityDeclNode entDecl) {

	myDecl = entDecl;

    } // end Constructor.

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	myDecl.loadDTDInfo();

    } // end method loadDIDInfo.

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("ENTITY ");
	myDecl.unparse(p, 0);

    } // end method unparse.

    // ----

    // 1 Kid.
    private EntityDeclNode myDecl;

} // end class EntitySpecNode.

// ========== NotationSpecNode ==========

class NotationSpecNode extends ContentNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public NotationSpecNode(IdNode name, StringLitNode value) {

	myName = name;
	myValue = value;

    } // end constructor.

    // ----

    public void loadDTDInfo() {}

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("NOTATION ");
	myName.unparse(p, 0);
	p.print(" PUBLIC ");
	myValue.unparse(p, 0);

    } // end method unparse.

    // ----

    // 2 kids.
    private IdNode myName;
    private StringLitNode myValue;

} // end class NotationSpecNode.

// ========== CommentNode ==========

class CommentNode extends ContentNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public CommentNode(int lineNum, int colNum, String strVal) {

        myLineNum = lineNum;
        myColNum = colNum;
        myComment = strVal;

    } // end Constructor.

    // ----

    public void loadDTDInfo() {}

    // ----
 
    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

        p.print(myComment);

    } // end method unparse.

    // ----
    
    private int myLineNum;    
    private int myColNum;   
    private String myComment;

} // end class CommentNode.

// -------------------------------------------------
// -=-=-=- EntityDeclNode and its subclasses -=-=-=-
// -------------------------------------------------

abstract class EntityDeclNode extends DTDnode
{
  public void loadDTDInfo() {};
}

// ========== ParamEntNode ==========

class ParamEntNode extends EntityDeclNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public ParamEntNode(ParsedNode parsedNode) {

	myParamEntity = parsedNode;

    } // end Constructr.

    // ----

    public void loadDTDInfo() {}

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("% ");
	myParamEntity.unparse(p, 0);

    } // end method unparse.

    // ----

    public String toString() {

	return myParamEntity.toString();

    } // end method toString.

    // ----

    // 1 kid.
    private ParsedNode myParamEntity;

} // end class ParamEntNode

// ========== ParsedNode ==========

class ParsedNode extends EntityDeclNode {

    /* ==========================================================
     *
     * Constructors.
     *
     * -------- */

    public ParsedNode(ParsedEntityDecl peDecl) {

	myParsedEnt = peDecl;
        haveNData = false;
	isExternal = true;

	// Create reduced string for this id.
	reducedTag = toBase26(++entCount);

    } // constructor 1.

    // ----

    public ParsedNode(ParsedEntityDecl peDecl, NDataNode nData) {

	myParsedEnt = peDecl;
	myNData = nData;
	haveNData = true;
	isExternal = true;

	// Create reduced string for this id.
	reducedTag = toBase26(++entCount);

    } // constructor 2.

    // ----

    /* ==========================================================
     *
     * method loadDTDInfo():
     * Find and store all elements, attributes, and entities found
     * from parsing the DTD.
     *
     * -------- */

    public void loadDTDInfo() {

	insertNode(2, this);

    } // end method loadDTDInfo.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	myParsedEnt.unparse(p, 0);
	if (haveNData) {
	  p.print(" ");
	  myNData.unparse(p, 0);
 	} // end if.

    } // end method unparse.

    // ----

    public String toString() {

	return myParsedEnt.toString();

    } // end method toString.

    // ----

    public String getNameId() {

	return (myParsedEnt.toString() + 
		" [" + reducedTag + "]");

    } // end method getNameId.

    // ----

    public String getId() {

	return (reducedTag);

    } // end method getId.

    // ----

    // 2 kids.
    private ParsedEntityDecl myParsedEnt;
    private NDataNode myNData;

    private String reducedTag;
    private boolean haveNData;
    private boolean isExternal;

} // end class ParsedNode

// ========== ParsedEntityDecl (and its subclasses) ==========

abstract class ParsedEntityDecl extends EntityDeclNode
{
}

// --------= ParsedPubNode =-----------

class ParsedPubNode extends ParsedEntityDecl {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public ParsedPubNode(IdNode entName, StringLitNode pubStr,

	StringLitNode secStr) {
	myName = entName;
	myPubStr = pubStr;
	mySecStr = secStr;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	myName.unparse(p, 0);
	p.print(" PUBLIC ");
	myPubStr.unparse(p, 0);
	p.print(" ");
	mySecStr.unparse(p, 0);

    } // end method unparse.

    // ----

    public String toString() {

	return myName.toString();

    } // end method toString.

    // ----

    // 3 Kids.
    private IdNode myName;
    private StringLitNode myPubStr;
    private StringLitNode mySecStr;
	
} // end class ParsedPubNode

// --------= ParsedSysNode =-----------

class ParsedSysNode extends ParsedEntityDecl {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public ParsedSysNode(IdNode entName, StringLitNode sysStr) {

	myName = entName;
	mySysStr = sysStr;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	myName.unparse(p, 0);
	p.print(" SYSTEM ");
	mySysStr.unparse(p, 0);

    } // end method unparse.

    // ----

    public String toString() {

	return myName.toString();

    } // end method toString.

    // ----

    // 2 Kids.
    private IdNode myName;
    private StringLitNode mySysStr;
	
} // end class ParsedSysNode.

// --------= ParsedPlainNode =---------

class ParsedPlainNode extends ParsedEntityDecl {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public ParsedPlainNode(IdNode name, StringLitNode strVal) {

	myName = name;
	myStr = strVal;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	myName.unparse(p, 0);
	p.print(" ");
	myStr.unparse(p, 0);

    } // end method unparse.

    // ----

    public String toString() {

	return myName.toString();

    } // end method toString.

    // ----

    // 2 kids.
    private IdNode myName;
    private StringLitNode myStr;

} // end class ParsedPlainNode.

// -------------------------------------------------
// -=-=-=- EltTypeNode and its subclasses -=-=-=-
// -------------------------------------------------

abstract class EltTypeNode extends DTDnode
{
  public void loadDTDInfo() {};
}

// ========== OtherNode ==========

class OtherNode extends EltTypeNode {

    /* ==========================================================
     *
     * Constructors.
     *
     * -------- */

    public OtherNode(IdNode someOther, OccurIndicNode occur) {

	myOther = someOther;
	myOccur = occur;
	haveContent = false;
	haveName = true;
	isEntity = false;

    } // end Constructor 1.

    // ----

    public OtherNode(IdNode otherName, OccurIndicNode occur,

	EltContentNode otherContent) {
	myOther = otherName;
	myOccur = occur;
	myOtherContent = otherContent;
	haveContent = true;
	haveName = true;
	isEntity = false;

    } // end Constructor 2.

    // ----

    public OtherNode(EntIdNode entId) {

	myEntId = entId;
	haveContent = false;
	haveName = false;
	isEntity = true;

    } // end Constructor 3.

    // ----

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	if (haveName)
	  myOther.unparse(p, 0);
//	p.print(reducedForm(myOther));
	if (haveContent) {
	  p.print("(");
	  myOtherContent.unparse(p, 0);
	  p.print(")");
	} // end if.
        if (isEntity)
	  myEntId.unparse(p, 0);
	else
	  myOccur.unparse(p, 0);

    } // end method unparse.

    // ----

    // 4 kids.
    private IdNode myOther;
    private EntIdNode myEntId;
    private OccurIndicNode myOccur;
    private EltContentNode myOtherContent;

    private boolean haveContent;
    private boolean haveName;
    private boolean isEntity;

} // end class OtherNode.

// ========== PCDataNode ==========

class PCDataNode extends EltTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public PCDataNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("#PCDATA");

    } // end method unparse.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class PCDataNode.

// ========== EmptyNode ==========

class EmptyNode extends EltTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public EmptyNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("EMPTY");

    } // end method unparse.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class EmptyNode.

// ========== AnyNode ==========

class AnyNode extends EltTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public AnyNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("ANY");

    } // end method unparse.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class AnyNode.

// -------------------------------------------------
// -=-=-=- AttrDefinitionNode and its subclasses -=-=-=-
// -------------------------------------------------

abstract class AttrDefinitionNode extends DTDnode
{
    abstract public String getType();
}

// ========== StdDefinitionNode ==========

class StdDefinitionNode extends AttrDefinitionNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public StdDefinitionNode(StdTypeNode tagType, StdDefaultNode def,
	StringLitNode initialVal) {

	myType = tagType;
	myDefault = def;
        if (initialVal.strVal().compareTo("NONE_GIVEN") != 0)
            myInitVal = initialVal.strVal();
        else
            myInitVal = "";

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	myType.unparse(p, 0);
	p.print(" ");
	myDefault.unparse(p, 0);
	p.print(" " + myInitVal);

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method getType():
     *
     * -------- */

    public String getType() {

	return myType.toString();

    } // end method getType.

    // ----

    // 2 kids.
    private StdTypeNode myType;;
    private StdDefaultNode myDefault;
    private String myInitVal;

} // end class StdDefinitionNode

// ========== EnumDefinitionNode ==========

class EnumDefinitionNode extends AttrDefinitionNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public EnumDefinitionNode(EnumTypeNode tagType,
		EnumDefaultNode def) {

	myType = tagType;
	myDefault = def;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	myType.unparse(p, 0);
	p.print(" ");
	myDefault.unparse(p, 0);

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method getType():
     *
     * -------- */

    public String getType() {

	return "";

    } // end method getType.

    // ----

    // 2 kids.
    private EnumTypeNode myType;
    private EnumDefaultNode myDefault;

} // end class EnumDefinitionNode.

// ========== NotationTypeNode ==========

 class NotationTypeNode extends AttrDefinitionNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public NotationTypeNode(EnumTypeNode tagType,
                EnumDefaultNode def) {

        myType = tagType;
        myDefault = def;

    } // end Constructor.

    // ----
 
    public void unparse(PrintWriter p, int indent) {

        p.print("NOTATION ");
        myType.unparse(p, 0);
        p.print(" ");
        myDefault.unparse(p, 0);

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method getType():
     *
     * -------- */

    public String getType() {

	return "";

    } // end method getType.

    // ----
 
    // 2 kids.
    private EnumTypeNode myType;
    private EnumDefaultNode myDefault;

} // end class NotationTypeNode.

// -------------------------------------------------
// -=-=-=- OccurIndicNode and its subclasses -=-=-=-
// -------------------------------------------------

abstract class OccurIndicNode extends DTDnode
{
}

// ========== PlusNode ==========

class PlusNode extends OccurIndicNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public PlusNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("+");

    } // end method unparse.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class PlusNode.

// ========== TimesNode ==========

class TimesNode extends OccurIndicNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public TimesNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("*");

    } // end method unparse.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class TimesNode.

// ========== QMarkNode ==========

class QMarkNode extends OccurIndicNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public QMarkNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("?");

    } // end method unparse.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class QMarkNode.

// ========== NothingNode ==========

class NothingNode extends OccurIndicNode {

    public NothingNode() { }
    public void unparse(PrintWriter p, int indent) { }

} // end class NothingNode.

// -------------------------------------------------
// -=-=-=- StdTypeNode and its subclasses -=-=-=-
// -------------------------------------------------

abstract class StdTypeNode extends DTDnode
{
}

// ========== CDataNode ==========

class CDataNode extends StdTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public CDataNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("CDATA");

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * -------- */

    public String toString() {

	return "CDATA";

    } // end method toString.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class CDataNode.

// ========== IdentNode ==========

class IdentNode extends StdTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public IdentNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end method Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("ID");

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * -------- */

    public String toString() {

	return "ID";

    } // end method toString.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class IdentNode.

// ========== IdRefNode ==========

class IdRefNode extends StdTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public IdRefNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("IDREF");

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * -------- */

    public String toString() {

	return "IDREF";

    } // end method toString.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class IdRefNode.

// ========== IdRefsNode ==========

class IdRefsNode extends StdTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public IdRefsNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("IDREFS");

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * -------- */

    public String toString() {

	return "IDREFS";

    } // end method toString.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class IdRefsNode.

// ========== EntityNode ==========

class EntityNode extends StdTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public EntityNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("ENTITY");

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * -------- */

    public String toString() {

	return "ENTITY";

    } // end method toString.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class EntityNode.

// ========== EntitiesNode ==========

class EntitiesNode extends StdTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public EntitiesNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("ENTITIES");

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * -------- */

    public String toString() {

	return "ENTITIES";

    } // end method toString.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class EntitiesNode.

// ========== NmTokenNode ==========

class NmTokenNode extends StdTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public NmTokenNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("NMTOKEN");

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * -------- */

    public String toString() {

	return "NMTOKEN";

    } // end method toString.

    // ----

    private int myLineNum;
    private int myColNum;

} // end NmTokenNode.

// ========== NmTokensNode ==========

class NmTokensNode extends StdTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public NmTokensNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("NMTOKENS");

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * -------- */

    public String toString() {

	return "NMTOKENS";

    } // end method toString.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class NmTokensNode.

// ========== UserDefinedTypeNode ==========

class UserDefinedTypeNode extends StdTypeNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public UserDefinedTypeNode(IdNode userType) {

	myType = userType;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("@");
	myType.unparse(p, indent);

    } // end method unparse.

    // ----

    /* ==========================================================
     *
     * method toString():
     *
     * -------- */

    public String toString() {

	return myType.toString();

    } // end method toString.

    // ----

    private IdNode myType;

} // end UserDefinedTypeNode.

// -------------------------------------------------
// -=-=-=- EnumDefaultNode and its subclasses -=-=-=-
// -------------------------------------------------

class EnumDefaultNode extends DTDnode {

    /* ==========================================================
     *
     * Constructors.
     *
     * -------- */

    public EnumDefaultNode() { }
    public EnumDefaultNode(StdDefaultNode def, StringLitNode value) {

	myDefault = def;
	myInitVal = value;

    } // end Constructor.

    // ----

    public void unparse(PrintWriter p, int indent) {

	if (myDefault.hasDefault)
	   myDefault.unparse(p, 0);
	if (myInitVal.strVal().compareTo("NONE_GIVEN") != 0)
	   p.print(" " + myInitVal.strVal());

    } // end method unparse.

    // ----

    // 2 kids
    private StdDefaultNode myDefault;
    private StringLitNode myInitVal;

} // end class EnumDefaultNode.

// ========== StdDefaultNode (and its subclasses) ==========

class StdDefaultNode extends EnumDefaultNode
{
    boolean hasDefault;
}

// --------= RequiredNode =---------

class RequiredNode extends StdDefaultNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public RequiredNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;
	hasDefault = true;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("#REQUIRED");

    } // end method unparse.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class RequiredNode.

// --------= FixedNode =---------

class FixedNode extends StdDefaultNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public FixedNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;
	hasDefault = true;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("#FIXED");

    } // end method unparse.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class FixedNode.

// --------= ImpliedNode =---------

class ImpliedNode extends StdDefaultNode {

    /* ==========================================================
     *
     * Constructor.
     *
     * -------- */

    public ImpliedNode(int lineNum, int colNum) {

	myLineNum = lineNum;
	myColNum = colNum;
	hasDefault = true;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method unparse():
     *
     * -------- */

    public void unparse(PrintWriter p, int indent) {

	p.print("#IMPLIED");

    } // end method unparse.

    // ----

    private int myLineNum;
    private int myColNum;

} // end class ImpliedNode.

// --------= NoDefaultNode =---------

class NoDefaultNode extends StdDefaultNode {

    public NoDefaultNode() { hasDefault = false; }
    public void unparse(PrintWriter p, int indent) { }

} // end class NoDefaultNode.

