// Armageddon Brown (armagedd)
// CS 764
// Prof. DeWitt

import java.io.*;
import java_cup.runtime.*;
import java.util.Random;

// **********************************************************************
// Main program to test the simple parser.
//
// There should be 2 command-line arguments:
//    1. the file to be parsed
//    2. the output file into which the AST built by the parser should be
//       unparsed
// The program opens the two files, creates a scanner and a parser, and
// calls the parser.  If the parse is successful, the AST is unparsed.
// **********************************************************************

public class dtdParse {

    public static void main(String[] args)
	throws IOException // may be thrown by the scanner
    {
	// check for command-line args
	if (args.length != 2) {
	    System.err.println("Usage: java dtdParse <dtd_file> <result_file>");
	    System.exit(-1);
	}

	// open input file
	FileReader inFile = null;
	try {
	    inFile = new FileReader(args[0]);
	} catch (FileNotFoundException ex) {
	    System.err.println("File " + args[0] + " not found.");
	    System.exit(-1);
	}

	// open output file for parsed dtd.
	PrintWriter outFile = null;
	try {
	    outFile = IO.openOutputFile("parsed.DTD");
	} catch (IOException ex) {
	    System.err.println("File 'parsed.DTD' could not be opened.");
	    System.exit(-1);
	}

	parser P = new parser(new Yylex(inFile));

	Symbol root=null; // the parser will return a Symbol whose value
	                  // field's type is the type associated with the
	                  // root nonterminal (i.e., with the nonterminal
	                  // "DTDDoc")

	try {
	    root = P.parse(); // do the parse
	    System.out.println ("\nDTD Document parsed correctly.\n" +
				"[ Parsed doc stored in 'parsed.DTD' ]\n");
	} catch (Exception ex){
	    System.out.println(ex);
	    System.exit(0);
	}
	((DTDnode)root.value).unparse(outFile, 0);
	outFile.close();

        Random rnd = new Random();
	DTDnode dtd = (DTDnode)root.value;
	dtd.loadDTDInfo();

	// -----------------==================------------------==============
	// ** New as of 06/2002 **.
	// -------------------------

	// re-open output file
	outFile = null;
	try {
	    outFile = IO.openOutputFile(args[1]);
	} catch (IOException ex) {
	    System.err.println("File " + args[1] + " could not be opened.");
	    System.exit(-1);
	}

	// Write out element and attribute names to a file.
	for (int i = 0; i < 2; i++) {
	    dtd.echoNodes(i,outFile);
	} // end for.
	outFile.close();

	// -----------------==================------------------==============

	int cnt = 0;
	while (cnt < 10) {
	    String elt = dtd.getRandomElement(rnd);
	    System.out.println(elt + ":\t" + dtd.getRandomAttribute(elt, rnd));
	    cnt++;
	} // end while.
	System.out.println(dtd.getRandomEntity(rnd) + "\n");
	cnt = 0;
	while (cnt++ < 8)
	    System.out.println(dtd.getRandomCDATA(rnd));

	System.out.println();
	return;
    }
}
