import java.util.*;

class Subtype {

    private String baseType;
    private Vector subtypes;

    // ----

    public Subtype(String declString) {

	int pos1, pos2;
	pos1 = declString.indexOf(" ");
	pos2 = declString.indexOf("ARE");
	if ((pos1 < 0) || (pos2 < 0)) {
	    System.out.println("-=- ERROR: Invalid subtype spec: "
		+ declString + " -=-");
	    baseType = "INVALID";
	    return;
	} // end if.

	// Initialize values.
	subtypes = new Vector();
	baseType = declString.substring(pos1, pos2);
	baseType = baseType.trim();
	if (baseType.length() == 0) {
	    System.out.println("-=- ERROR: Invalid subtype"
		+ "(no base type). -=-");
	    baseType = "INVALID";
	    return;
	} // end if.

	pos1 = declString.indexOf(".");
        String delims = "\t ,";
	declString = declString.substring(pos2+3, pos1);
        StringTokenizer tokenizer = new StringTokenizer(declString, delims);
        String tmpStr;
	while (tokenizer.hasMoreTokens()) {
	    tmpStr = tokenizer.nextToken();
	    addSubtype(tmpStr);
	} // end while.

	// Finally, treat "NONE" as a subset of EVERYTHING (i.e. make it
	// compatible with all other types, so it has no effect).
	addSubtype("NONE");

	// Done.
	return;

    } // end Constructor.

    // ----

    /* ==========================================================
     *
     * method getAllSubtypesIn():
     *
     * Takes a vector of constraints, and returns a new vector
     * with all of those constraints, PLUS any constraints that
     * are subtypes of those constraints, by using the received
     * 'subtypes' vector.
     *
     * -------- */

    public static Vector getAllSubtypesIn(Vector constraints,
	Vector subtypes) {

	String constraint;
	Subtype sub;
	Vector result = new Vector();
	result.addAll(constraints);
	Iterator itr = constraints.iterator();
	while (itr.hasNext()) {
	    constraint = (String)itr.next();
	    Iterator itr2 = subtypes.iterator();
	    while (itr2.hasNext()) {
		sub = (Subtype)itr2.next();
		if (sub.getBase().equals(constraint)) {
		// then we have subtypes for the current constraint;
		// add them to our list.
		    result.addAll(sub.allSubTypes());
		    break;
		} // end if.
	    } // end while.
	} // end while.

	// Finally, return the full list of subtypes.
	return result;

    } // end method getAllSubtypesIn.

    // ----

    /* ==========================================================
     *
     * method getDeclFor():
     *
     * Return the Subtype object corresponding to the 'base' type.
     * The returned Subtype must exist in the received 'subtypes'
     * vector.
     *
     * -------- */

    public static Subtype getDeclFor(String base, Vector subtypes) {

	Subtype sub;
	Iterator itr = subtypes.iterator();
	while (itr.hasNext()) {
	    sub = (Subtype)itr.next();
	    if (sub.getBase().equals(base)) {
	    // then we found it.
		return sub;
	    } // end if.
	} // end while.

	// If we get here, we didn't find a match.
	return null;

    } // end method getDeclFor.
 
    // ----

    public void addSubtype(String subtype) {

	subtypes.add(subtype.trim());

    } // end method addSubtype.

    // ----

    public boolean matches(String aType) {

	return (baseType.equals(aType) ||
	    subtypes.contains(aType));

    } // end method matches.

    // ----

    public String getBase() {

	return baseType;

    } // end method getBase.

    // ----

    public Vector allSubTypes() {

	return subtypes;

    } // end method getAllSubtypes.

    // ----

    public String toString() {

	return (baseType + " == " + subtypes);

    } // end method toString.

} // end class Subtype.
