import java.io.*;
import java.lang.Math;
import java.util.*;

// -----------------============== NOTE =============--------------------
// To make changes to "terminal" data (for grammars other than XQuery),
// go to the "Generator.java" file--that is the ONLY file that needs to
// be modified for those purposes; everything else is designed to run
// independently of the grammar being used.
// -----------------=================================--------------------

// ----------------------------------------
// -=-=-=-=-=- TokenContentNode -=-=-=-=-=-
// ----------------------------------------

class TokenContentNode extends GrammarNode {

    public TokenContentNode(DefListNode defList) {

	myDefList = defList;
	type = DEF_LIST;

    } // end constructor 1.

    // ----

    public TokenContentNode(CharSetNode charSet) {

	myCharSet = charSet;
	type = CHARSET;

    } // end constructor 2.

    // ----

    public TokenContentNode(StringLitNode str) {

	myStr = str;
	type = STRING_LIT;

    } // end constructor 3.

    // ----

    public TokenContentNode(IdNode id) {

	myId = id;
	type = ID;

    } // end constructor 4.

    // ----

    public void unparse(PrintWriter p, int indent) {

	switch (type) {
	    case DEF_LIST:	p.print("(");
				myDefList.unparse(p, indent);
				p.print(")");
				break;
	    case CHARSET:	myCharSet.unparse(p, indent);
				myCharSet.createCharList();
				break;
	    case STRING_LIT:	myStr.unparse(p, indent);
				break;
	    case ID:		myId.unparse(p, indent);
				break;
	    default:		p.print("UNKNOWN_TOKEN_CONTENT");
				break;
	} // end switch.

    } // end method unparse.

    // ----

    public String charSelect() {

        String str = myCharSet.charSet();
        if (str.length() == 0) {
        // then an invalid char set was read.
            return "BAD_CHAR!";
        } // end if.
 
        int which = generator.getRandom().nextInt(str.length());
        return ("" + str.charAt(which));

    } // end method charSelect.

    // ----

    public String toString() {

	switch (type) {

	    case DEF_LIST:	return ("(" + myDefList.toString() + ")");
	    case CHARSET:	return myCharSet.toString();
	    case STRING_LIT:	return "\"" + myStr.toString() + "\"";
	    case ID:		return myId.toString();
	    default:		return "CONTENT_UNAVAILABLE";

	} // end switch.

    } // end method toString.

    // ----

    public int type() {

	return type;

    } // end method type.

    // ----

    public DefListNode getList() {

	return myDefList;

    } // end method getList.

    // ----

    // 4 Kids.
    private StringLitNode myStr;
    private IdNode myId;
    private CharSetNode myCharSet;
    private DefListNode myDefList;
    private int type;

} // end class TokenContentNode.
