import java_cup.runtime.*; // defines the Symbol class
// The generated scanner will return a Symbol for each token that it finds.
// A Symbol contains an Object field named value; that field will be of type
// TokenVal, defined below.
//
// A TokenVal object contains the line number on which the token occurs as
// well as the number of the character on that line that starts the token.
// Some tokens (e.g., literals) also include the value of the token.
class TokenVal {
 // fields
    int linenum;
    int charnum;
 // constructor
    TokenVal(int l, int c) {
        linenum = l;
	charnum = c;
    }
}
class IdTokenVal extends TokenVal {
 // new field: the id name
    String idVal;
 // constructor
    IdTokenVal( int l, int c, String value ) {
     	super( l, c );
	idVal = value;
    }
}
class StrLitTokenVal extends TokenVal {
 // new field: the actual representation of the string literal.
    String strVal;
 // constructor
    StrLitTokenVal(int l, int c, String theString) {
	super(l,c);
	strVal = theString;
    }
}
class CharSetTokenVal extends TokenVal {
 // new field: the actual representation of the string literal.
    String strVal;
 // constructor
    CharSetTokenVal(int l, int c, String theString) {
	super(l,c);
	strVal = theString;
    }
}
// The following class is used to keep track of the character number at which
// the current token starts on its line.
class CharNum {
  static int num=1;
}


public class Yylex implements java_cup.runtime.Scanner {
	private final int YY_BUFFER_SIZE = 512;
	private final int YY_F = -1;
	private final int YY_NO_STATE = -1;
	private final int YY_NOT_ACCEPT = 0;
	private final int YY_START = 1;
	private final int YY_END = 2;
	private final int YY_NO_ANCHOR = 4;
	private final int YY_BOL = 128;
	private final int YY_EOF = 129;
	private java.io.BufferedReader yy_reader;
	private int yy_buffer_index;
	private int yy_buffer_read;
	private int yy_buffer_start;
	private int yy_buffer_end;
	private char yy_buffer[];
	private int yyline;
	private boolean yy_at_bol;
	private int yy_lexical_state;

	public Yylex (java.io.Reader reader) {
		this ();
		if (null == reader) {
			throw (new Error("Error: Bad input stream initializer."));
		}
		yy_reader = new java.io.BufferedReader(reader);
	}

	public Yylex (java.io.InputStream instream) {
		this ();
		if (null == instream) {
			throw (new Error("Error: Bad input stream initializer."));
		}
		yy_reader = new java.io.BufferedReader(new java.io.InputStreamReader(instream));
	}

	private Yylex () {
		yy_buffer = new char[YY_BUFFER_SIZE];
		yy_buffer_read = 0;
		yy_buffer_index = 0;
		yy_buffer_start = 0;
		yy_buffer_end = 0;
		yyline = 0;
		yy_at_bol = true;
		yy_lexical_state = YYINITIAL;
	}

	private boolean yy_eof_done = false;
	private final int YYINITIAL = 0;
	private final int yy_state_dtrans[] = {
		0
	};
	private void yybegin (int state) {
		yy_lexical_state = state;
	}
	private int yy_advance ()
		throws java.io.IOException {
		int next_read;
		int i;
		int j;

		if (yy_buffer_index < yy_buffer_read) {
			return yy_buffer[yy_buffer_index++];
		}

		if (0 != yy_buffer_start) {
			i = yy_buffer_start;
			j = 0;
			while (i < yy_buffer_read) {
				yy_buffer[j] = yy_buffer[i];
				++i;
				++j;
			}
			yy_buffer_end = yy_buffer_end - yy_buffer_start;
			yy_buffer_start = 0;
			yy_buffer_read = j;
			yy_buffer_index = j;
			next_read = yy_reader.read(yy_buffer,
					yy_buffer_read,
					yy_buffer.length - yy_buffer_read);
			if (-1 == next_read) {
				return YY_EOF;
			}
			yy_buffer_read = yy_buffer_read + next_read;
		}

		while (yy_buffer_index >= yy_buffer_read) {
			if (yy_buffer_index >= yy_buffer.length) {
				yy_buffer = yy_double(yy_buffer);
			}
			next_read = yy_reader.read(yy_buffer,
					yy_buffer_read,
					yy_buffer.length - yy_buffer_read);
			if (-1 == next_read) {
				return YY_EOF;
			}
			yy_buffer_read = yy_buffer_read + next_read;
		}
		return yy_buffer[yy_buffer_index++];
	}
	private void yy_move_end () {
		if (yy_buffer_end > yy_buffer_start &&
		    '\n' == yy_buffer[yy_buffer_end-1])
			yy_buffer_end--;
		if (yy_buffer_end > yy_buffer_start &&
		    '\r' == yy_buffer[yy_buffer_end-1])
			yy_buffer_end--;
	}
	private boolean yy_last_was_cr=false;
	private void yy_mark_start () {
		int i;
		for (i = yy_buffer_start; i < yy_buffer_index; ++i) {
			if ('\n' == yy_buffer[i] && !yy_last_was_cr) {
				++yyline;
			}
			if ('\r' == yy_buffer[i]) {
				++yyline;
				yy_last_was_cr=true;
			} else yy_last_was_cr=false;
		}
		yy_buffer_start = yy_buffer_index;
	}
	private void yy_mark_end () {
		yy_buffer_end = yy_buffer_index;
	}
	private void yy_to_mark () {
		yy_buffer_index = yy_buffer_end;
		yy_at_bol = (yy_buffer_end > yy_buffer_start) &&
		            ('\r' == yy_buffer[yy_buffer_end-1] ||
		             '\n' == yy_buffer[yy_buffer_end-1] ||
		             2028/*LS*/ == yy_buffer[yy_buffer_end-1] ||
		             2029/*PS*/ == yy_buffer[yy_buffer_end-1]);
	}
	private java.lang.String yytext () {
		return (new java.lang.String(yy_buffer,
			yy_buffer_start,
			yy_buffer_end - yy_buffer_start));
	}
	private int yylength () {
		return yy_buffer_end - yy_buffer_start;
	}
	private char[] yy_double (char buf[]) {
		int i;
		char newbuf[];
		newbuf = new char[2*buf.length];
		for (i = 0; i < buf.length; ++i) {
			newbuf[i] = buf[i];
		}
		return newbuf;
	}
	private final int YY_E_INTERNAL = 0;
	private final int YY_E_MATCH = 1;
	private java.lang.String yy_error_string[] = {
		"Error: Internal error.\n",
		"Error: Unmatched input.\n"
	};
	private void yy_error (int code,boolean fatal) {
		java.lang.System.out.print(yy_error_string[code]);
		java.lang.System.out.flush();
		if (fatal) {
			throw new Error("Fatal Error.\n");
		}
	}
	private int[][] unpackFromString(int size1, int size2, String st) {
		int colonIndex = -1;
		String lengthString;
		int sequenceLength = 0;
		int sequenceInteger = 0;

		int commaIndex;
		String workString;

		int res[][] = new int[size1][size2];
		for (int i= 0; i < size1; i++) {
			for (int j= 0; j < size2; j++) {
				if (sequenceLength != 0) {
					res[i][j] = sequenceInteger;
					sequenceLength--;
					continue;
				}
				commaIndex = st.indexOf(',');
				workString = (commaIndex==-1) ? st :
					st.substring(0, commaIndex);
				st = st.substring(commaIndex+1);
				colonIndex = workString.indexOf(':');
				if (colonIndex == -1) {
					res[i][j]=Integer.parseInt(workString);
					continue;
				}
				lengthString =
					workString.substring(colonIndex+1);
				sequenceLength=Integer.parseInt(lengthString);
				workString=workString.substring(0,colonIndex);
				sequenceInteger=Integer.parseInt(workString);
				res[i][j] = sequenceInteger;
				sequenceLength--;
			}
		}
		return res;
	}
	private int yy_acpt[] = {
		/* 0 */ YY_NOT_ACCEPT,
		/* 1 */ YY_NO_ANCHOR,
		/* 2 */ YY_NO_ANCHOR,
		/* 3 */ YY_NO_ANCHOR,
		/* 4 */ YY_NO_ANCHOR,
		/* 5 */ YY_NO_ANCHOR,
		/* 6 */ YY_NO_ANCHOR,
		/* 7 */ YY_NO_ANCHOR,
		/* 8 */ YY_NO_ANCHOR,
		/* 9 */ YY_NO_ANCHOR,
		/* 10 */ YY_NO_ANCHOR,
		/* 11 */ YY_NO_ANCHOR,
		/* 12 */ YY_NO_ANCHOR,
		/* 13 */ YY_NO_ANCHOR,
		/* 14 */ YY_NO_ANCHOR,
		/* 15 */ YY_NO_ANCHOR,
		/* 16 */ YY_NOT_ACCEPT,
		/* 17 */ YY_NO_ANCHOR,
		/* 18 */ YY_NO_ANCHOR,
		/* 19 */ YY_NO_ANCHOR,
		/* 20 */ YY_NOT_ACCEPT,
		/* 21 */ YY_NO_ANCHOR,
		/* 22 */ YY_NOT_ACCEPT,
		/* 23 */ YY_NO_ANCHOR,
		/* 24 */ YY_NOT_ACCEPT,
		/* 25 */ YY_NOT_ACCEPT,
		/* 26 */ YY_NOT_ACCEPT,
		/* 27 */ YY_NOT_ACCEPT,
		/* 28 */ YY_NOT_ACCEPT,
		/* 29 */ YY_NOT_ACCEPT,
		/* 30 */ YY_NO_ANCHOR
	};
	private int yy_cmap[] = unpackFromString(1,130,
"14:9,2,1,14:21,2,14,10,14:4,12,19,20,17,16,14,7:2,14,6:10,5,14,3,9,8,18,14," +
"4:26,13,14,11,14,4,14,4:26,14,15,14:3,0:2")[0];

	private int yy_rmap[] = unpackFromString(1,31,
"0,1,2:2,3,1:6,4,1:3,5,6,7,1,5,8,1,9,10,11,12,13,14,15,16,17")[0];

	private int yy_nxt[][] = unpackFromString(18,21,
"1,2,3,16,4:2,-1:4,20,-1,22,24,-1,5,6,7,8,9,10,-1:22,3:2,-1:19,25:2,-1,4,30," +
"4:2,21,-1:20,18,-1:13,15:2,-1:22,17:2,-1:4,20,-1:14,17:4,21,-1:13,20:9,11,2" +
"0:10,-1,22:11,12,22:8,-1,25:2,-1,4,23,4:2,21,14,-1:12,26:5,27,26:4,-1,26:9," +
"-1,25:2,-1:2,28,-1:16,26:10,13,26:9,-1,26:5,27,26:4,19,26:9,-1:5,29,-1:24,1" +
"4,-1:12,25:2,-1,4,23,4:2,21,-1:12");

	public java_cup.runtime.Symbol next_token ()
		throws java.io.IOException {
		int yy_lookahead;
		int yy_anchor = YY_NO_ANCHOR;
		int yy_state = yy_state_dtrans[yy_lexical_state];
		int yy_next_state = YY_NO_STATE;
		int yy_last_accept_state = YY_NO_STATE;
		boolean yy_initial = true;
		int yy_this_accept;

		yy_mark_start();
		yy_this_accept = yy_acpt[yy_state];
		if (YY_NOT_ACCEPT != yy_this_accept) {
			yy_last_accept_state = yy_state;
			yy_mark_end();
		}
		while (true) {
			if (yy_initial && yy_at_bol) yy_lookahead = YY_BOL;
			else yy_lookahead = yy_advance();
			yy_next_state = YY_F;
			yy_next_state = yy_nxt[yy_rmap[yy_state]][yy_cmap[yy_lookahead]];
			if (YY_EOF == yy_lookahead && true == yy_initial) {

return new Symbol(sym.EOF);
			}
			if (YY_F != yy_next_state) {
				yy_state = yy_next_state;
				yy_initial = false;
				yy_this_accept = yy_acpt[yy_state];
				if (YY_NOT_ACCEPT != yy_this_accept) {
					yy_last_accept_state = yy_state;
					yy_mark_end();
				}
			}
			else {
				if (YY_NO_STATE == yy_last_accept_state) {
					throw (new Error("Lexical Error: Unmatched Input."));
				}
				else {
					yy_anchor = yy_acpt[yy_last_accept_state];
					if (0 != (YY_END & yy_anchor)) {
						yy_move_end();
					}
					yy_to_mark();
					switch (yy_last_accept_state) {
					case 1:
						
					case -2:
						break;
					case 2:
						{ CharNum.num = 1;}
					case -3:
						break;
					case 3:
						{CharNum.num += yytext().length();}
					case -4:
						break;
					case 4:
						{ 
	    // Found an identifier.
	    // If we have a "<" before our ID or a ">" after, we
	    // just want to ignore it.
	    int pos1 = yytext().indexOf("<") + 1;
	    if (pos1 < 0)
	    // then we don't have a left one.
		pos1 = 0;
	    int pos2 = yytext().indexOf(">");
	    if (pos2 < 0)
	    // then we don't have a right one.
		pos2 = yytext().length();
	    Symbol S = new Symbol( 
			sym.ID, 
			new IdTokenVal( yyline + 1, CharNum.num, yytext().substring(pos1,pos2) ) 
	    );
	    CharNum.num += yytext().length();
	    return S;
	   }
					case -5:
						break;
					case 5:
						{// VBAR token.
            Symbol S = new Symbol(sym.VBAR,
                                  new TokenVal(yyline+1, CharNum.num));
            CharNum.num += yytext().length();
            return S;
           }
					case -6:
						break;
					case 6:
						{// PLUS token.
            Symbol S = new Symbol(sym.PLUS,
                                  new TokenVal(yyline+1, CharNum.num));
            CharNum.num += yytext().length();
            return S;
           }
					case -7:
						break;
					case 7:
						{// STAR token.
            Symbol S = new Symbol(sym.STAR,
                                  new TokenVal(yyline+1, CharNum.num));
            CharNum.num += yytext().length();
            return S;
           }
					case -8:
						break;
					case 8:
						{// QMARK token.
            Symbol S = new Symbol(sym.QMARK,
                                  new TokenVal(yyline+1, CharNum.num));
            CharNum.num += yytext().length();
            return S;
           }
					case -9:
						break;
					case 9:
						{// LPAR token.
            Symbol S = new Symbol(sym.LPAR,
                                  new TokenVal(yyline+1, CharNum.num));
            CharNum.num += yytext().length();
            return S;
           }
					case -10:
						break;
					case 10:
						{// RPAR token.
            Symbol S = new Symbol(sym.RPAR,
                                  new TokenVal(yyline+1, CharNum.num));
            CharNum.num += yytext().length();
            return S;
           }
					case -11:
						break;
					case 11:
						{// LEGAL STRING LITERAL
	    // If we have a "<" before our string or a ">" after, we
	    // just want to ignore it.
	    int pos1 = yytext().indexOf("<") + 1;
	    if (pos1 != 1)
	    // if it's not the first character (i.e. BEFORE the open
	    // quote), then keep it.
		pos1 = 0;
	    int pos2 = yytext().indexOf(">");
	    if (pos2 != yytext().length()-1)
	    // if it's not the last character (i.e. AFTER the close
	    // quote), then keep it.
		pos2 = yytext().length();
            Symbol S = new Symbol(sym.STRINGLITERAL,
                                  new StrLitTokenVal(yyline+1, CharNum.num,
                                                     yytext().substring(pos1,pos2)));
            CharNum.num += yytext().length();
            return S;
           }
					case -12:
						break;
					case 12:
						{// LEGAL STRING LITERAL
            Symbol S = new Symbol(sym.STRINGLITERAL,
                                  new StrLitTokenVal(yyline+1, CharNum.num,
                                                     yytext()));
            CharNum.num += yytext().length();
            return S;
           }
					case -13:
						break;
					case 13:
						{ // Legal Char Set.
            Symbol S = new Symbol(sym.CHARSET,
                                  new CharSetTokenVal(yyline+1, CharNum.num,
                                                     yytext()));
            CharNum.num += yytext().length();
            return S;
           }
					case -14:
						break;
					case 14:
						{ // Found the "head" of a rule..
		Symbol S = new Symbol( 
			sym.HEAD,
			new IdTokenVal( yyline + 1, CharNum.num, yytext() ) 
		);
		CharNum.num += yytext().length();
		return S;
	   }
					case -15:
						break;
					case 15:
						{ // Legal Rule Number.
            Symbol S = new Symbol(sym.RULENUM,
                                  new TokenVal(yyline+1, CharNum.num));
            CharNum.num += yytext().length();
            return S;
           }
					case -16:
						break;
					case 17:
						{ 
	    // Found an identifier.
	    // If we have a "<" before our ID or a ">" after, we
	    // just want to ignore it.
	    int pos1 = yytext().indexOf("<") + 1;
	    if (pos1 < 0)
	    // then we don't have a left one.
		pos1 = 0;
	    int pos2 = yytext().indexOf(">");
	    if (pos2 < 0)
	    // then we don't have a right one.
		pos2 = yytext().length();
	    Symbol S = new Symbol( 
			sym.ID, 
			new IdTokenVal( yyline + 1, CharNum.num, yytext().substring(pos1,pos2) ) 
	    );
	    CharNum.num += yytext().length();
	    return S;
	   }
					case -17:
						break;
					case 18:
						{// LEGAL STRING LITERAL
	    // If we have a "<" before our string or a ">" after, we
	    // just want to ignore it.
	    int pos1 = yytext().indexOf("<") + 1;
	    if (pos1 != 1)
	    // if it's not the first character (i.e. BEFORE the open
	    // quote), then keep it.
		pos1 = 0;
	    int pos2 = yytext().indexOf(">");
	    if (pos2 != yytext().length()-1)
	    // if it's not the last character (i.e. AFTER the close
	    // quote), then keep it.
		pos2 = yytext().length();
            Symbol S = new Symbol(sym.STRINGLITERAL,
                                  new StrLitTokenVal(yyline+1, CharNum.num,
                                                     yytext().substring(pos1,pos2)));
            CharNum.num += yytext().length();
            return S;
           }
					case -18:
						break;
					case 19:
						{ // Legal Char Set.
            Symbol S = new Symbol(sym.CHARSET,
                                  new CharSetTokenVal(yyline+1, CharNum.num,
                                                     yytext()));
            CharNum.num += yytext().length();
            return S;
           }
					case -19:
						break;
					case 21:
						{ 
	    // Found an identifier.
	    // If we have a "<" before our ID or a ">" after, we
	    // just want to ignore it.
	    int pos1 = yytext().indexOf("<") + 1;
	    if (pos1 < 0)
	    // then we don't have a left one.
		pos1 = 0;
	    int pos2 = yytext().indexOf(">");
	    if (pos2 < 0)
	    // then we don't have a right one.
		pos2 = yytext().length();
	    Symbol S = new Symbol( 
			sym.ID, 
			new IdTokenVal( yyline + 1, CharNum.num, yytext().substring(pos1,pos2) ) 
	    );
	    CharNum.num += yytext().length();
	    return S;
	   }
					case -20:
						break;
					case 23:
						{ 
	    // Found an identifier.
	    // If we have a "<" before our ID or a ">" after, we
	    // just want to ignore it.
	    int pos1 = yytext().indexOf("<") + 1;
	    if (pos1 < 0)
	    // then we don't have a left one.
		pos1 = 0;
	    int pos2 = yytext().indexOf(">");
	    if (pos2 < 0)
	    // then we don't have a right one.
		pos2 = yytext().length();
	    Symbol S = new Symbol( 
			sym.ID, 
			new IdTokenVal( yyline + 1, CharNum.num, yytext().substring(pos1,pos2) ) 
	    );
	    CharNum.num += yytext().length();
	    return S;
	   }
					case -21:
						break;
					case 30:
						{ 
	    // Found an identifier.
	    // If we have a "<" before our ID or a ">" after, we
	    // just want to ignore it.
	    int pos1 = yytext().indexOf("<") + 1;
	    if (pos1 < 0)
	    // then we don't have a left one.
		pos1 = 0;
	    int pos2 = yytext().indexOf(">");
	    if (pos2 < 0)
	    // then we don't have a right one.
		pos2 = yytext().length();
	    Symbol S = new Symbol( 
			sym.ID, 
			new IdTokenVal( yyline + 1, CharNum.num, yytext().substring(pos1,pos2) ) 
	    );
	    CharNum.num += yytext().length();
	    return S;
	   }
					case -22:
						break;
					default:
						yy_error(YY_E_INTERNAL,false);
					case -1:
					}
					yy_initial = true;
					yy_state = yy_state_dtrans[yy_lexical_state];
					yy_next_state = YY_NO_STATE;
					yy_last_accept_state = YY_NO_STATE;
					yy_mark_start();
					yy_this_accept = yy_acpt[yy_state];
					if (YY_NOT_ACCEPT != yy_this_accept) {
						yy_last_accept_state = yy_state;
						yy_mark_end();
					}
				}
			}
		}
	}
}
