// Armageddon Brown (armagedd)
// CS 764
// Prof. DeWitt

import java.io.*;
import java_cup.runtime.*;
import java.util.*;

// import edu.wisc.cs.ragx.dtd.*;

/* Main RAGX class.  Reads and loads all input files, parses the input
 * grammar, and then generates sentences based on that grammar, printing
 * the results to screen. */

public class ragx {

    public static final int NUM_QUERIES = 10;

    public static void main(String[] args)
	throws IOException // may be thrown by the scanner
    {

	// ====----
	// ****** Check the command line args. ******
	// ====----

	if (args.length < 7) {
	    System.err.println("\nUsage:\n------\n\n" +
		" java ragx <terminal_data_file>\n" +
		"\t\t<grammar_file>\n\t\t<config_file> <overwrite(y/n)>\n" +
		"\t\t<types_file>\n\t\t<op_tables_file>\n\t\t<complexity>\n\n" +
		"\t--> Use 'none' as <terminal_data_file> if there isn't one.\n" +
		"\t--> 'complexity' should be an integer between 1 and 10.\n" +
		"\t--> Use y/n to say whether or not the config file should be\n" +
		"\t\toverwritten (with a default one).\n");
	    System.exit(-1);
	}

	// ====----
	// ****** Open and Load 'terminal' data. ******
	// ====----
	// [ 'Terminal' data in the case of XQuery grammars refers to the
	// [ element and attribute id's pulled out of a parsed DTD; this
	// [ data is stored in some file, and has to be loaded into memory.
	// [ Note that Elt/Attr id's are specific to the XQuery grammar
	// [ generator.  For other grammars, terminal data will be different.
	//
	// [ ##### ALL changes with respect to terminal data should be	#####
	// [ ##### done in the "Generator.java" file, and ONLY in that	#####
	// [ ##### file!!						#####
	//
        // (Everything else is designed to run independently of the
        // grammar being used.)
	//
	// ====----

	Object terminalData = null;
	FileReader inFile = null;
	if (!(args[0].equals("none"))) {
	// only do this if we actually have terminal data.

	    // -= Open =-
	    try {
		inFile = new FileReader(args[0]);
	    } catch (FileNotFoundException ex) {
		System.err.println("File " + args[0] + " not found.");
		System.exit(-1);
	    } // end try-catch.

	    // -= Load =-
	    terminalData = Generator.loadTerminalData(inFile);
	    inFile.close();

	} // end if.

	// ====----
	// ****** Open grammar file for parsing. ******
	// ====----

	inFile = null;
	try {
	    inFile = new FileReader(args[1]);
	} catch (FileNotFoundException ex) {
	    System.err.println("File " + args[1] + " not found.");
	    System.exit(-1);
	} // end try-catch.

	// ====----
	// ****** Open "parsed.gram" file for parsed result. ******
	// ====----

	PrintWriter outFile = null;
	try {
	    outFile = IO.openOutputFile("parsed.gram");
	} catch (IOException ex) {
	    System.err.println("File 'parsed.gram' could not be opened.");
	    System.exit(-1);
	} // end try-catch.

	// ====----
	// ****** Parse the grammar and store the result. ******
	// ====----

	System.out.println("\nParsing grammar file...");
	parser P = new parser(new Yylex(inFile));
	Symbol root = null;	// the parser will return a Symbol whose value
	                  	// field's type is the type associated with the
	                  	// root nonterminal (i.e., with the nonterminal
	                  	// "GrammarNode")
	try {
	    root = P.parse(); // do the parse
	    System.out.println("--> Grammar parsed correctly. <--\n" +
		"[ Result stored in 'parsed.gram' ]\n");
	} catch (Exception ex) {
	    System.out.println(ex);
	    System.exit(0);
	} // end try.
	((GrammarNode)root.value).unparse(outFile, 0);
	outFile.close();

	// ====----
	// ****** Load config file ******
	// ====----

	boolean newConfig;
	if (args[3].equals("y") || args[3].equals("Y")) {
	// then we want to create a NEW config file (overwrite old one).
	    newConfig = true;
	} // end if.
	else {
	// use the whatever configuration file exists.
	    newConfig = false;
	} // end else.

	try {
	    if (newConfig) {
		outFile = IO.openOutputFile(args[2]);
		((GrammarNode)root.value).createConfigFile(outFile);
		outFile.close();
	    } // end if.
	    FileReader configFile = new FileReader(args[2]);
	    ((GrammarNode)root.value).loadConfigFile(configFile, args[6]);
	} catch (IOException ex) {
		System.err.println("\n-=- ERROR: File \"" + args[2] + 
		    "\" could not be opened. -=-\n");
	    System.exit(-1);
	} // end catch.

	// ====----
	// ****** Load types file ******
	// ====----

	FileReader typesFile = null;
	try {
	    typesFile = new FileReader(args[4]);
	    ((GrammarNode)root.value).loadTypesFile(typesFile);
	} catch (IOException ex) {
	    System.err.println("\n-=- ERROR: File " + args[4] +
			" not found. -=-\n");
	    System.exit(-1);
	} // end try-catch.

	// ====----
	// ****** Load operation tables from file ******
	// ====----

	FileReader opsFile = null;
	try {
	    opsFile = new FileReader(args[5]);
	    ((GrammarNode)root.value).loadOpTables(opsFile);
	} catch (IOException ex) {
	    System.err.println("\n-=- ERROR: File " + args[5] +
			" not found. -=-\n");
	    System.exit(-1);
	} // end try-catch.

	// ====----
	// ****** Finally, generate the results. ******
	// ====----

	GrammarNode gram = (GrammarNode)root.value;
	gram.setTerminals(terminalData);
	GResult res = null;
	String query = "";
	for (int i = 0; i < NUM_QUERIES; i++) {
	    System.out.println("\n----------===== " + i + " =====-----------");
	    res = ((RuleNode)gram.rules.firstElement()).generate(0, null);
	    query = res.strVal;
	    System.out.println(query);
	    gram.resetCount();
	} // end for.

	System.out.println("\nDone.\n");
	return;

    } // end main.

    // ----

    /* ==========================================================
     *
     * method readLine():
     * Reads a line from the received FileReader and returns it.
     *
     * -------- */

    static String readLine(FileReader aFile) {
 
	String str = "";
	try {
	    int tmpChar = aFile.read();
	    if (tmpChar == -1) {
		return "EOF";
	    } // end if.
	    while ((tmpChar != -1) && ((char)tmpChar != '\n')) {
		str += (char)tmpChar;
		tmpChar = aFile.read();
	    } // end while.
	} // end try.
	catch (IOException e) {
	    e.printStackTrace();
	} // end catch.

	return str;
 
    } // end method readLine.

} // end class ragx.
